-- Extended Wood Harvester Cutting
-- Specialization to automatically cut trees in specified lengths
-- Version 1.0.0.0
-- @author Vector Man
-- @date 2023-01-06
-- copyright (c) Vector Man, All Rights Reserved.

source(g_currentModDirectory .. "src/gui/CutLengthSettingsMenu.lua")
source(g_currentModDirectory .. "src/events/UpdateCutLengthsEvent.lua")

ExtendedWoodHarvesterCutting = {
	prerequisitesPresent = function (specializations)
		return SpecializationUtil.hasSpecialization(WoodHarvester, specializations)
	end,
	
	MaxAmountCuttingLengths = 20,
	ModName = g_currentModName,
	ModDirectory = g_currentModDirectory
}
ExtendedWoodHarvesterCutting.specName = string.format("spec_%s.extendedWoodHarvesterCutting", g_currentModName)

function ExtendedWoodHarvesterCutting.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "inputAction_OPEN_GUI", ExtendedWoodHarvesterCutting.inputAction_OPEN_GUI)
	SpecializationUtil.registerFunction(vehicleType, "inputAction_AUTOMATIC_CUT", ExtendedWoodHarvesterCutting.inputAction_AUTOMATIC_CUT)
	SpecializationUtil.registerFunction(vehicleType, "cutNextStep", ExtendedWoodHarvesterCutting.cutNextStep)
	SpecializationUtil.registerFunction(vehicleType, "openCutLengthSettingsGui", ExtendedWoodHarvesterCutting.openCutLengthSettingsGui)
	SpecializationUtil.registerFunction(vehicleType, "updateCutLengths", ExtendedWoodHarvesterCutting.updateCutLengths)
	SpecializationUtil.registerFunction(vehicleType, "startAutomaticCutting", ExtendedWoodHarvesterCutting.startAutomaticCutting)
	SpecializationUtil.registerFunction(vehicleType, "getCutLengths", ExtendedWoodHarvesterCutting.getCutLengths)
end

function ExtendedWoodHarvesterCutting.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", ExtendedWoodHarvesterCutting)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", ExtendedWoodHarvesterCutting)
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", ExtendedWoodHarvesterCutting)
	SpecializationUtil.registerEventListener(vehicleType, "onCutTree", ExtendedWoodHarvesterCutting)
end

function ExtendedWoodHarvesterCutting:initSpecialization()
		local schemaSavegame = Vehicle.xmlSchemaSavegame

		for i = 1, ExtendedWoodHarvesterCutting.MaxAmountCuttingLengths do
			schemaSavegame:register(XMLValueType.FLOAT, "vehicles.vehicle(?)." .. ExtendedWoodHarvesterCutting.ModName .. string.format(".extendedWoodHarvesterCutting.autoCutLengths.piece%s#length", i), string.format("Cut length for %s. piece", i), true)
		end
end

function ExtendedWoodHarvesterCutting:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.isClient then
		local spec = self.spec_extendedWoodHarvesterCutting
		self:clearActionEventsTable(spec.actionEvents)
		
		-- Remove conflicting binding: FOLD_ALL_IMPLEMENTS (key_lctrl key_x) also triggers wood harvester unfolding and blocks the automatic mode input action
		if self.spec_foldable ~= nil then
			self:removeActionEvent(self.spec_foldable.actionEvents, InputAction.FOLD_ALL_IMPLEMENTS)self:removeActionEvent(spec.actionEvents, InputAction.FOLD_ALL_IMPLEMENTS)
		end
		if self.spec_attachable ~= nil then
			if self.spec_attachable:getAttacherVehicle() ~= nil then
				if self.spec_attachable:getAttacherVehicle().spec_foldable ~= nil then
					self:removeActionEvent(self.spec_attachable:getAttacherVehicle().spec_foldable.actionEvents, InputAction.FOLD_ALL_IMPLEMENTS)
				end
			end
		end

		if isActiveForInputIgnoreSelection then
			local actionEventId
			_, actionEventId = self:addActionEvent(spec.actionEvents, "OPEN_GUI", self, ExtendedWoodHarvesterCutting.inputAction_OPEN_GUI, false, true, false, true, true, nil )
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_LOW)
			g_inputBinding:setActionEventTextVisibility(actionEventId, false)
			_, actionEventId = InputBinding.registerActionEvent(g_inputBinding, "OPEN_GUI", self, ExtendedWoodHarvesterCutting.inputAction_OPEN_GUI, false, true, false, true)
			
			_, actionEventId = self:addPoweredActionEvent(spec.actionEvents, "AUTOMATIC_CUT", self, ExtendedWoodHarvesterCutting.inputAction_AUTOMATIC_CUT, true, true, false, true, true, nil )
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_LOW)
			g_inputBinding:setActionEventTextVisibility(actionEventId, false)
			_, actionEventId = InputBinding.registerActionEvent(g_inputBinding, "AUTOMATIC_CUT", self, ExtendedWoodHarvesterCutting.inputAction_AUTOMATIC_CUT, true, true, false, true)
		end
	end
end

function ExtendedWoodHarvesterCutting:onLoad(savegame)
	-- Create spec table in self
    self.spec_extendedWoodHarvesterCutting = self[ExtendedWoodHarvesterCutting.specName]
	local spec = self.spec_extendedWoodHarvesterCutting
	local specWoodHarvester = self.spec_woodHarvester
	
	spec.isCuttingAllowed = false
	spec.isAutomaticRunning = false
	spec.cutLengths = {}
	spec.currentStep = 1
	spec.cooldownTimer = -1.0
	spec.rememberedCutLength = 0
	
	local lengthValue
	for i = 1, ExtendedWoodHarvesterCutting.MaxAmountCuttingLengths do
		if savegame ~= nil then
			lengthValue = savegame.xmlFile:getValue(savegame.key .. "." .. ExtendedWoodHarvesterCutting.ModName .. string.format(".extendedWoodHarvesterCutting.autoCutLengths.piece%s#length", i))
		end
		if lengthValue == nil then
			lengthValue = specWoodHarvester.currentCutLength
		end
		table.insert(spec.cutLengths, lengthValue)
	end
end

function ExtendedWoodHarvesterCutting:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	local spec = self.spec_extendedWoodHarvesterCutting
	local specWoodHarvester = self.spec_woodHarvester
	
	if spec.actionEvents["AUTOMATIC_CUT"] ~= nil then
		if specWoodHarvester.hasAttachedSplitShape then
			g_inputBinding:setActionEventTextVisibility(spec.actionEvents["AUTOMATIC_CUT"].actionEventId, true)
			spec.isCuttingAllowed = true
		else
			g_inputBinding:setActionEventTextVisibility(spec.actionEvents["AUTOMATIC_CUT"].actionEventId, false)
			spec.isCuttingAllowed = false
			spec.currentStep = 1
		end
		
		if spec.cooldownTimer > 0.0 then
			spec.cooldownTimer = spec.cooldownTimer - 0.004 * dt
		elseif spec.cooldownTimer > -1.0 then
			if not self:getIsAnimationPlaying(specWoodHarvester.cutAnimation.name) then
				spec.cooldownTimer = -1.0
				self:cutNextStep()
			end
		end
	end
	
	if not specWoodHarvester.automaticCuttingEnabled and specWoodHarvester.automaticCuttingIsDirty and spec.isCuttingAllowed and spec.isAutomaticRunning and not specWoodHarvester.isAttachedSplitShapeMoving then
		self:cutTree(0)
	end
	
	if not specWoodHarvester.hasAttachedSplitShape and spec.wasNewTree then
		spec.wasNewTree = false
		specWoodHarvester.currentCutLength = spec.rememberedCutLength
	end
end

function ExtendedWoodHarvesterCutting:cutNextStep()
	local spec = self.spec_extendedWoodHarvesterCutting
	local specWoodHarvester = self.spec_woodHarvester

	if spec.isCuttingAllowed and spec.isAutomaticRunning and not specWoodHarvester.isAttachedSplitShapeMoving then
		if not specWoodHarvester.automaticCuttingEnabled and specWoodHarvester.automaticCuttingIsDirty then
			self:cutTree(0)
		else
			if spec.currentStep <= #spec.cutLengths then
				specWoodHarvester.currentCutLength = spec.cutLengths[spec.currentStep]
			else
				specWoodHarvester.currentCutLength = spec.cutLengths[#spec.cutLengths]
			end
			self:cutTree(specWoodHarvester.currentCutLength)
			spec.currentStep = spec.currentStep + 1
		end
	end
end

function ExtendedWoodHarvesterCutting:onCutTree()
	local spec = self.spec_extendedWoodHarvesterCutting
	
	spec.cooldownTimer = 1.0
end
	
function ExtendedWoodHarvesterCutting:openCutLengthSettingsGui()
	local spec = self.spec_extendedWoodHarvesterCutting
	
	g_cutLengthSettingsMenu.modDirectory = ExtendedWoodHarvesterCutting.ModDirectory
	g_gui:showDialog("cutLengthSettingsMenu", true)
end

function ExtendedWoodHarvesterCutting:updateCutLengths(lengths, noEventSend)
	local spec = self.spec_extendedWoodHarvesterCutting
	
	UpdateCutLengthsEvent.sendEvent(self, lengths, noEventSend)
	
	if self.isClient then
		spec.cutLengths = lengths
	end
end

function ExtendedWoodHarvesterCutting:getCutLengths()
	return self.spec_extendedWoodHarvesterCutting.cutLengths
end

function ExtendedWoodHarvesterCutting:startAutomaticCutting(inputValue, noEventSend)
	local spec = self.spec_extendedWoodHarvesterCutting
	local specWoodHarvester = self.spec_woodHarvester
	
	if not spec.wasNewTree then
		spec.rememberedCutLength = specWoodHarvester.currentCutLength
		spec.wasNewTree = true
	end
	
	if inputValue > 0 then
		if spec.isCuttingAllowed then
			spec.isAutomaticRunning = true
			self:cutNextStep()
			g_inputBinding:setActionEventActive(specWoodHarvester.actionEvents["IMPLEMENT_EXTRA2"], false)
		end
	else
		spec.isAutomaticRunning = false
		g_inputBinding:setActionEventActive(specWoodHarvester.actionEvents["IMPLEMENT_EXTRA2"], true)
	end
end

function ExtendedWoodHarvesterCutting:inputAction_OPEN_GUI(actionName, inputValue, callbackState, isAnalog)
	self:openCutLengthSettingsGui()
end

function ExtendedWoodHarvesterCutting:inputAction_AUTOMATIC_CUT(actionName, inputValue, callbackState, isAnalog)
	self:startAutomaticCutting(inputValue, false)
end

function ExtendedWoodHarvesterCutting:saveToXMLFile(xmlFile, key, usedModNames)
	local spec = self.spec_extendedWoodHarvesterCutting

	for i = 1, #spec.cutLengths do
		setXMLFloat(xmlFile.handle, key .. string.format(".autoCutLengths.piece%s#length", i), spec.cutLengths[i])
	end
end