CutLengthSettingsMenu = {}
CutLengthSettingsMenu.CONTROLS = {
    "backButton",
    "saveButton",
}

local CutLengthSettingsMenu_mt = Class(CutLengthSettingsMenu, ScreenElement)
local texts = {}
local modDirectory = nil

function CutLengthSettingsMenu.new(target, custom_mt)
	local self = DialogElement.new(target, custom_mt or CutLengthSettingsMenu_mt)

	self:registerControls(CutLengthSettingsMenu.CONTROLS)
	return self
end

function CutLengthSettingsMenu:update(dt)
	CutLengthSettingsMenu:superClass().update(self, dt)
	
	local vehicle = g_currentMission.controlledVehicle
	if vehicle.spec_woodHarvester == nil and vehicle.spec_attacherJoints ~= nil then
		for _,attachedImplement in pairs(vehicle.spec_attacherJoints:getAttachedImplements()) do
			if attachedImplement.object.spec_woodHarvester ~= nil then
				vehicle = attachedImplement.object
				break
			end
		end
	end
	
	if vehicle.spec_extendedWoodHarvesterCutting == nil then
		self:onClickBack()
	else
		local pointerPositions = self:ewhcGetValues()
		for i = 1, 20 do
			-- pointers
			if i ~= 1 then
				if tostring(pointerPositions[i]) ~= "inf" then
					self.elements[string.format("ewhcPointer%s", i)].element:setPosition(self.elements[string.format("ewhcPointer%s", i-1)].element.position[1] - 0.02 * pointerPositions[i], -0.04)
				else
					self.elements[string.format("ewhcPointer%s", i)].element:setPosition(self.elements[string.format("ewhcPointer%s", i-1)].element.position[1] - 10, -0.04)
				end
			else
				if tostring(pointerPositions[i]) ~= "inf" then
					self.elements[string.format("ewhcPointer%s", i)].element:setPosition(-0.02 * pointerPositions[i], -0.04)
				else
					self.elements[string.format("ewhcPointer%s", i)].element:setPosition(-10, -0.04)
				end
			end
			
			local visible
			if self.elements[string.format("ewhcPointer%s", i)].element.position[1] < -0.4 then
				visible = false
			else
				visible = true
			end
			self.elements[string.format("ewhcPointer%s", i)].element:setVisible(visible)
			
			-- texts
			if tostring(pointerPositions[1]) == "inf" then
				self.elements["ewhcVisualizationText1"].element:setText(g_i18n:getText("case_wholeTree"))
				self.elements["ewhcVisualizationText1"].element:setPosition(-0.058, -0.04)
			else
				self.elements[string.format("ewhcVisualizationText%s", i)].element:setText(pointerPositions[i] .. "m")
				local p1, p2
				if i == 1 then
					p1 = 0.285
				else
					p1 = self.elements[string.format("ewhcPointer%s", i-1)].element.position[1] + 0.285
				end
				p2 = self.elements[string.format("ewhcPointer%s", i)].element.position[1]
				self.elements[string.format("ewhcVisualizationText%s", i)].element:setPosition(p1 + 0.5 * (p2 - p1), -0.08)
			end
			if i > 1 then
				if self.elements[string.format("ewhcPointer%s", i-1)].element.visible and self.elements[string.format("ewhcPointer%s", i)].element.visible then
					self.elements[string.format("ewhcVisualizationText%s", i)].element:setVisible(true)
				else
					self.elements[string.format("ewhcVisualizationText%s", i)].element:setVisible(false)
				end
			else
				self.elements[string.format("ewhcVisualizationText%s", i)].element:setVisible(true)
			end
		end
	end
end

function CutLengthSettingsMenu:onCreate()
end

function CutLengthSettingsMenu:onOpen()
	CutLengthSettingsMenu:superClass().onOpen(self)
	
	local vehicle = g_currentMission.controlledVehicle
	if vehicle.spec_woodHarvester == nil and vehicle.spec_attacherJoints ~= nil then
		for _,attachedImplement in pairs(vehicle.spec_attacherJoints:getAttachedImplements()) do
			if attachedImplement.object.spec_woodHarvester ~= nil then
				vehicle = attachedImplement.object
				break
			end
		end
	end
	
	self.ewhcInputAllowed = true
	
	for _, t in pairs(vehicle.spec_woodHarvester.cutLengths) do
		if tostring(t) ~= "inf" then
			table.insert(texts, string.format("%sm", tostring(t)))
		else
			table.insert(texts, tostring(t))
		end
		
	end
	
	self:ewhcLoadValues()
	
	-- log overlay
	self.elements["ewhcLogOverlay"].element:setImageFilename(self.modDirectory .. "textures/gui/gui_ewhcSettingIcons.png")
	
	-- vehicle icon
	local vehicleStoreIcon = vehicle:getImageFilename()
	self.elements["ewhcVehicleIcon"].element:setImageFilename(vehicleStoreIcon)
	
	-- vehicle text
	self.elements["ewhcVehicleText"].element:setText(string.format(g_i18n:getText("settingsMenuVehicleText"), vehicle:getFullName()))
	
	-- header icon
	self.elements["ewhcHeaderIcon"].element:setImageFilename(self.modDirectory .. "textures/gui/gui_ewhcSettingIcons.png")
	
	-- pointer icons
	for i = 1, 20 do
		self.elements[string.format("ewhcPointer%s", i)].element:setImageFilename(self.modDirectory .. "textures/gui/gui_ewhcSettingIcons.png")
	end
	
	return false
end 

function CutLengthSettingsMenu:onClickBack()
	self:ewhcSaveValues(false)
	self:ewhcResetValues()
	self.ewhcInputAllowed = false

	self:changeScreen(nil)
end 

function CutLengthSettingsMenu:ewhcLoadValues()
	local vehicle = g_currentMission.controlledVehicle
	if vehicle.spec_woodHarvester == nil and vehicle.spec_attacherJoints ~= nil then
		for _,attachedImplement in pairs(vehicle.spec_attacherJoints:getAttachedImplements()) do
			if attachedImplement.object.spec_woodHarvester ~= nil then
				vehicle = attachedImplement.object
				break
			end
		end
	end

	local elementId = 1
	local i = 1
	while elementId <= #vehicle:getCutLengths() do
		local key = "piece"
		if elementId < 10 then
			key = key .. "0"
		end
		key = key .. tostring(elementId)
		
		if i <= #vehicle:getCutLengths() then
			if self.elements[key].element ~= nil then
				if self.elements[key].element.typeName == "multiTextOption" and i == elementId then
					self.elements[key].element:setTexts(texts)
					local state = 1
					while state <= #texts do
						if texts[state] ~= "inf" then
							if string.sub(texts[state], 1, -2) == tostring(vehicle:getCutLengths()[elementId]) then
								break
							end
						else
							if texts[state] == tostring(vehicle:getCutLengths()[elementId]) then
								break
							end
						end
						state = state + 1
					end
					self.elements[key].element:setState(state)
					elementId = elementId + 1
					i = 0
				end
			end
			i = i + 1
		else
			break
		end
	end
end

function CutLengthSettingsMenu:ewhcGetValues()
	local vehicle = g_currentMission.controlledVehicle
	if vehicle.spec_woodHarvester == nil and vehicle.spec_attacherJoints ~= nil then
		for _,attachedImplement in pairs(vehicle.spec_attacherJoints:getAttachedImplements()) do
			if attachedImplement.object.spec_woodHarvester ~= nil then
				vehicle = attachedImplement.object
				break
			end
		end
	end
	
	local ret = {}

	local elementId = 1
	local i = 1
	while elementId <= #vehicle:getCutLengths() do
		local key = "piece"
		if elementId < 10 then
			key = key .. "0"
		end
		key = key .. tostring(elementId)
		
		if i <= #vehicle:getCutLengths() then
			if self.elements[key].element ~= nil then
				if self.elements[key].element.typeName == "multiTextOption" and i == elementId then
					local state = self.elements[key].element:getState()
					if self.elements[key].element.texts[state] ~= "inf" then
						table.insert(ret, tonumber(string.sub(self.elements[key].element.texts[state], 1, -2)))
					else
						table.insert(ret, tonumber(self.elements[key].element.texts[state]))
					end
					elementId = elementId + 1
					i = 0
				end
			end
			i = i + 1
		else
			break
		end
	end
	return ret
end

function CutLengthSettingsMenu:ewhcSaveValues()
	local vehicle = g_currentMission.controlledVehicle
	if vehicle.spec_woodHarvester == nil and vehicle.spec_attacherJoints ~= nil then
		for _,attachedImplement in pairs(vehicle.spec_attacherJoints:getAttachedImplements()) do
			if attachedImplement.object.spec_woodHarvester ~= nil then
				vehicle = attachedImplement.object
				break
			end
		end
	end
	vehicle:updateCutLengths(self:ewhcGetValues(), false)
end

function CutLengthSettingsMenu:ewhcResetValues()
	local vehicle = g_currentMission.controlledVehicle
	if vehicle.spec_woodHarvester == nil and vehicle.spec_attacherJoints ~= nil then
		for _,attachedImplement in pairs(vehicle.spec_attacherJoints:getAttachedImplements()) do
			if attachedImplement.object.spec_woodHarvester ~= nil then
				vehicle = attachedImplement.object
				break
			end
		end
	end

	local elementId = 1
	local i = 1
	while elementId <= #vehicle:getCutLengths() do
		local key = "piece"
		if elementId < 10 then
			key = key .. "0"
		end
		key = key .. tostring(elementId)
		
		if i <= #vehicle:getCutLengths() then
			if self.elements[key].element ~= nil then
				if self.elements[key].element.typeName == "multiTextOption" and i == elementId then
					self.elements[key].element:setTexts({})
					self.elements[key].element:setState(1)
					elementId = elementId + 1
					i = 0
				end
			end
			i = i + 1
		else
			break
		end
	end
	
	texts = {}
end

function CutLengthSettingsMenu:onCreateSubElement(element)
	self.elements[element.id] = {element = element}
end